DROP VIEW IF EXISTS access_review_details_v;
DROP VIEW IF EXISTS adhoc_suser_v;
DROP VIEW IF EXISTS suser_spermission_cert_risk_v;
DROP VIEW IF EXISTS adhoc_apps_spermissions_v;
DROP VIEW IF EXISTS review_item_status_v;
DROP VIEW IF EXISTS shared_accounts_v;
DROP VIEW IF EXISTS suser_v;
DROP VIEW IF EXISTS adhoc_sgroup_suser_v;

CREATE VIEW shared_accounts_v AS 
  SELECT 
    acct.unique_account_id,
    saccu.suser,
    suser.display_name,
    substring(split_part(suser.dn, ',', 1), 4) AS user_cn
  FROM saccount acct
    JOIN saccount_user saccu ON acct.collected_entity = saccu.saccount AND acct.relation_to_user_type = 'SHARED' AND acct.effective = '1'
    JOIN suser suser ON suser.collected_entity = saccu.suser AND suser.effective = '1' AND suser.deleted = '0' AND saccu.effective = '1';

CREATE VIEW review_item_status_v AS 
  SELECT 
    perm.id perm_id,
    suser.id user_id,
    item.status review_status,
    max(item.end_date) AS last_certification
  FROM spermission perm
    JOIN (SELECT
        item.unique_permission_id,
        item.unique_user_id,
        review_instance.id,
        review_instance.status,
        review_instance.end_date
      FROM review_item item 
        JOIN review_instance ON item.review_instance_id = review_instance.id AND review_instance.status IN ('CERTIFIED', 'CERTIFYING', 'AUDIT_APPROVAL')
      WHERE item.unique_permission_id IS NOT NULL AND item.unique_user_id IS NOT NULL) item ON item.unique_permission_id = perm.unique_permission_id AND perm.effective = true
    JOIN suser ON suser.unique_user_id = item.unique_user_id AND suser.effective = true
  GROUP BY perm.id, suser.id, item.status;

CREATE VIEW adhoc_apps_spermissions_v AS 
  SELECT 
    suser_account_permission.suser,
    suser_account_permission.saccount,
    data_source.id AS application_id,
    data_source.name AS application_name,
    data_source.description AS application_description,
    application.risk AS application_risk,
    spermission.id AS permission_id,
    spermission.name AS permission_name,
    spermission.description AS permission_description,
    spermission.risk AS permission_risk,
    spermission.type AS permission_type,
    certified.last_certification,
      CASE
        WHEN certified.last_certification IS NOT NULL THEN true
        ELSE false
      END AS is_certified,
      CASE
        WHEN certified.review_status IN ('CERTIFYING', 'AUDIT_APPROVAL') THEN true
        ELSE false
      END AS certification_pending
  FROM suser_account_permission
     JOIN spermission ON suser_account_permission.spermission = spermission.id AND spermission.deleted = false AND spermission.effective = true
     JOIN application ON spermission.application = application.id
     JOIN data_source ON application.id = data_source.id AND data_source.data_source_type = 'APPLICATION' AND data_source.deleted = false
     LEFT JOIN review_item_status_v certified ON suser_account_permission.suser = certified.user_id AND spermission.id = certified.perm_id;

CREATE VIEW suser_spermission_cert_risk_v AS 
  SELECT 
    spermissions.suser AS suser_id, 
    coalesce(spermissions.is_certified, false) AS is_certified, 
    coalesce(spermissions.certification_pending, false) AS certification_pending, 
    coalesce(sum(spermissions.permission_risk), 0) AS aggregate_permission_risk, 
    count(spermissions.suser) AS certification_count 
  FROM adhoc_apps_spermissions_v spermissions
  GROUP BY spermissions.suser, spermissions.is_certified, spermissions.certification_pending, spermissions.permission_risk;

CREATE VIEW adhoc_suser_v AS 
  SELECT
    suser.id,
    suser.deleted,
    suser.department,
    suser.display_description,
    suser.display_name,
    suser.dn,
    suser.effective,
    suser.employee_status,
    suser.employee_type,
    suser.first_name,
    suser.full_name,
    suser.guid,
    suser.home_phone,
    suser.idm_dn,
    suser.idm_tree_name,
    suser.is_curated,
    suser.job_code,
    suser.job_title,
    suser.last_name,
    suser.location,
    suser.merged,
    suser.middle_name,
    suser.office_phone,
    suser.other_phone,
    suser.preferred_locale,
    suser.provisioning_id,
    suser.secondary_supervisor_id,
    suser.supervisor_attr_key,
    suser.supervisor_id,
    suser.supervisor_unique_id,
    suser.unique_user_id,
    suser.user_id,
    suser.business_unit,
    suser.collected_entity,
    suser.effective_entity,
    suser.identity_source,
    suser.sversion,
    suser.workforce_id,
    suser_email.email,
    suser.risk,
    cert_risk.certification_count,
    cert_risk.is_certified,
    cert_risk.certification_pending,
    cert_risk.aggregate_permission_risk,
    permissions.all_permissions
  FROM suser
    LEFT JOIN suser_email ON suser.id = suser_email.suser
    LEFT JOIN (SELECT 
        sap.suser,
        string_agg(perm.name, ' | ' ORDER BY perm.name) AS all_permissions
      FROM spermission perm
        JOIN suser_account_permission sap ON perm.id = sap.spermission
      GROUP BY sap.suser) permissions ON suser.id = permissions.suser
    LEFT JOIN suser_spermission_cert_risk_v cert_risk ON suser.id = cert_risk.suser_id
  WHERE suser.deleted = false AND suser.effective = true;

CREATE VIEW access_review_details_v AS
  SELECT DISTINCT 
    item.id AS review_item_id,
    item.last_override_action_id AS last_override_action,
    instance.id AS review_instance_id,
    instance.status AS review_status,
    instance.start_date,
    instance.end_date,
    instance.valid_to_date,
    instance.item_count,
    multi_owner.review_owners,
    multi_reviewer.reviewer_ids AS reviewer_id,
        CASE
            WHEN item.final_cert_by_unique_user_id IS NOT NULL THEN final_reviewer.display_name
            ELSE multi_reviewer.reviewers
        END AS reviewer_display_name,
        CASE
            WHEN item.final_cert_by_unique_user_id IS NOT NULL THEN substring(split_part(final_reviewer.dn, ',', 1), 4)
            ELSE multi_reviewer.user_cn
        END AS reviewer_cn,
    reassigned_by_history.reviewers AS reassigned_by_user_disp_name,
    reassigned_from_history.reviewers AS reassigned_from_user_disp_name,
    reassigned_by_history.reassigned_date,
    reassigned_by_history.reasons AS reassigned_reasons,
    reassigned_by_history.rev_comments reassigned_comments,
    defn.name AS review_name,
    substr(defn.description, 0, 4000) AS description,
    defn.review_type,
    defn.next_schedule_date,
        CASE
            WHEN item.item_type = 'ACCOUNT' AND usr.id IS NULL THEN shacc.suser
            ELSE usr.id
        END AS user_id,
        CASE
            WHEN item.item_type = 'ACCOUNT' AND usr.display_name IS NULL THEN shacc.display_name
            ELSE usr.display_name
        END AS user_display_name,
        CASE
            WHEN item.item_type = 'ACCOUNT' AND usr.display_name IS NULL THEN shacc.user_cn
            ELSE substring(split_part(usr.dn, ',', 1), 4)
        END AS user_cn,
    action.action AS reviewer_action,
    reason.name AS modify_reason_name,
    reason.id AS modify_reason_id,
        CASE
            WHEN fulfillment.review_item_id IS NOT NULL THEN true
            ELSE false
        END AS remove_completed,
    usr.unique_user_id,
    action.review_order,
    action.reviewer_comment,
    perm.id AS permission_id,
    perm.name AS permission_name,
    acct.id AS account_id,
    acct.account_id AS account_native_id,
    acct.name AS account_name,
    acct.relation_to_user_type AS account_relation,
    app.id AS application_id,
    app_ds.name AS application_name,
    item.final_decision,
    item.final_decision_time,
    role.id AS role_id,
    role.name AS role_name,
    item.item_type,
    brole.name AS brole_name,
    action.fulfillment_instructions,
    esc_reviewer.display_name esc_reviewer
   FROM review_item item
     JOIN review_instance instance ON item.review_instance_id = instance.id AND instance.status NOT IN ('PREVIEW', 'PREVIEW_CANCELED', 'PREVIEW_STARTING')
     JOIN (SELECT
          review_instance.id inst_id,
          defn.id defn_id, 
          CASE WHEN staged_defn.name IS NULL THEN defn.name ELSE staged_defn.name END AS name,
          CASE WHEN staged_defn.description IS NULL THEN defn.description ELSE staged_defn.description END AS description,
          CASE WHEN staged_defn.next_schedule_date IS NULL THEN defn.next_schedule_date ELSE staged_defn.next_schedule_date END AS next_schedule_date,
          defn.review_type,
          defn.deleted
        FROM review_definition defn
        JOIN review_instance ON defn.id = review_instance.review_definition
        LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn ON instance.id = defn.inst_id
      LEFT JOIN suser usr ON usr.unique_user_id = item.unique_user_id AND usr.effective = true
      LEFT JOIN spermission perm ON perm.unique_permission_id = item.unique_permission_id AND perm.effective = true
      LEFT JOIN saccount acct ON acct.unique_account_id = item.unique_account_id AND acct.effective = true
      LEFT JOIN application app ON app.id =
        CASE
            WHEN perm.id IS NULL THEN acct.application
            ELSE perm.application
        END
      LEFT JOIN business_role brole ON brole.id = item.business_role_id
      LEFT JOIN role_policy role ON role.id = item.role_id
      LEFT JOIN data_source app_ds ON app_ds.id = app.id
      LEFT JOIN review_item_action action ON action.review_item_id = item.id AND action.action <> 'REASSIGNED'
      LEFT JOIN review_item_action_reason reason ON reason.id = action.reason_id
      LEFT JOIN reviewer_instance inst2 ON instance.id = inst2.review_instance_id AND inst2.type = 'RO'
      LEFT JOIN reviewer_instance inst3 ON instance.id = inst3.review_instance_id
      LEFT JOIN ( SELECT DISTINCT 
          item_1.id AS item_id,
          string_agg(multi_reviewers.id::text, ' | ' ORDER BY multi_reviewers.display_name) AS reviewer_ids,
          string_agg(multi_reviewers.unique_user_id, ' | ' ORDER BY (multi_reviewers.display_name)) AS reviewer_unique_ids,
          string_agg(multi_reviewers.display_name, ' | ' ORDER BY (multi_reviewers.display_name)) AS reviewers,
          string_agg(substring(split_part(multi_reviewers.dn, ',', 1), 4), ' | ' ORDER BY (multi_reviewers.display_name)) AS user_cn
        FROM review_item item_1
          LEFT JOIN reviewer_user_instance rrui ON rrui.reviewer_instance_id = item_1.curr_reviewer_instance_id
          LEFT JOIN suser multi_reviewers ON rrui.unique_user_id = multi_reviewers.unique_user_id AND multi_reviewers.effective = true
          GROUP BY item_1.id) multi_reviewer ON multi_reviewer.item_id = item.id
      LEFT JOIN suser final_reviewer ON item.final_cert_by_unique_user_id = final_reviewer.unique_user_id AND final_reviewer.effective = true
      LEFT JOIN suser acting_reviewer ON action.acted_by_user_unique_id = acting_reviewer.unique_user_id AND acting_reviewer.effective = true
      LEFT JOIN review_item_action reassigned ON reassigned.review_item_id = item.id AND reassigned.action = 'REASSIGNED' AND reassigned.reassigned_to_user_unique_id ~~ (('%' || multi_reviewer.reviewer_unique_ids) || '%')
      LEFT JOIN suser curr_reviewer ON curr_reviewer.unique_user_id ~~ (('%' || multi_reviewer.reviewer_unique_ids) || '%') AND curr_reviewer.effective = true
      LEFT JOIN ( SELECT 
          ria.review_item_id,
          string_agg(usr_1.display_name, ' | ' ORDER BY ria.assigned) AS reviewers
        FROM review_item_action ria
          LEFT JOIN reviewer_instance ri ON ri.id = ria.reviewer_instance_id
          LEFT JOIN suser usr_1 ON ri.single_reviewer_unique_id = usr_1.unique_user_id AND usr_1.effective = true
        WHERE ria.action = 'REASSIGNED'
        GROUP BY ria.review_item_id) reassigned_from_history ON reassigned_from_history.review_item_id = item.id
        LEFT JOIN (SELECT 
            ria.review_item_id,
            string_agg(usr_1.display_name, ' | ' ORDER BY ria.assigned) AS reviewers,
            string_agg(to_char(ria.assigned, 'YYYY-MM-DD HH24:MI:SS'), ' | ' ORDER BY ria.assigned) AS reassigned_date,
            string_agg(COALESCE(reason_1.name, 'NULL_REASON'), ' | ' ORDER BY ria.assigned) AS reasons,
            string_agg(COALESCE (NULLIF(ria.reviewer_comment, ''), 'NULL_COMMENT'), ' | ' ORDER BY ria.assigned) AS rev_comments
          FROM review_item_action ria
            LEFT JOIN review_item_action_reason reason_1 ON ria.reason_id = reason_1.id
            LEFT JOIN suser usr_1 ON ria.reassigned_by_user_unique_id = usr_1.unique_user_id AND usr_1.effective = true
          WHERE ria.action = 'REASSIGNED'
          GROUP BY ria.review_item_id) reassigned_by_history ON reassigned_by_history.review_item_id = item.id
        LEFT JOIN shared_accounts_v shacc ON shacc.unique_account_id = acct.unique_account_id
        LEFT JOIN ( SELECT DISTINCT 
            defn_1.name AS review_name,
            ri.id AS rid,
            string_agg(s.display_name, ' | ' ORDER BY (s.display_name)) AS review_owners
          FROM review_instance ri
            JOIN review_definition defn_1 ON ri.review_definition = defn_1.id
            LEFT JOIN reviewer_instance rev_owner ON ri.id = rev_owner.review_instance_id AND rev_owner.type = 'RO'
            LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner.id
            LEFT JOIN suser s ON rui.unique_user_id = s.unique_user_id AND s.effective = true
          GROUP BY defn_1.name, ri.id) multi_owner ON multi_owner.rid = item.review_instance_id
        LEFT JOIN ( SELECT 
            esc_reviewers.review_instance_id,
            string_agg(esc_reviewers.display_name, ' | ' ORDER BY (esc_reviewers.display_name)) AS display_name
          FROM ( SELECT DISTINCT 
              u.display_name,
              rri.review_instance_id
            FROM review_item ri
              JOIN reviewer_instance rri ON ri.review_instance_id = rri.review_instance_id AND rri.type = 'EO'
              JOIN reviewer_user_instance rriu ON rriu.reviewer_instance_id = rri.id
              JOIN suser u ON u.unique_user_id = rriu.unique_user_id AND u.effective = true) esc_reviewers
            GROUP BY esc_reviewers.review_instance_id) esc_reviewer ON esc_reviewer.review_instance_id = item.review_instance_id
        LEFT JOIN ( SELECT 
            cri.review_item_id,
            cia.action,
            cia.change_item_status
          FROM change_request_item cri
            LEFT JOIN change_item_action cia ON cri.latest_action = cia.id
          WHERE cri.change_request_type ~~ ('REMOVE' || '%') AND cia.action = 'FULFILLED' AND (cia.change_item_status = 'VERIFIED' OR cia.change_item_status = 'COMPLETED')) fulfillment ON item.id = fulfillment.review_item_id
  WHERE
      defn.deleted = false
      AND (acting_reviewer.unique_user_id ~~ (('%' || multi_reviewer.reviewer_unique_ids) || '%') OR acting_reviewer.unique_user_id IS NULL 
          AND action.review_order = 
          ( SELECT DISTINCT min(act.review_order) AS min
             FROM review_item_action act
             WHERE (act.action = 'PENDING' OR act.action = 'ESCALATE') AND act.review_item_id = item.id) 
          OR acting_reviewer.unique_user_id IS NOT NULL AND action.shared_queue = true) 
      AND (action.action <> 'PENDING' OR item.final_decision IS NULL);

CREATE VIEW suser_v AS 
  SELECT
    suser.id,
    suser.deleted,
    suser.department,
    suser.display_description,
    suser.display_name,
    suser.dn,
    suser.effective,
    suser.employee_status,
    suser.employee_type,
    suser.first_name,
    suser.full_name,
    suser.guid,
    suser.home_phone,
    suser.idm_dn,
    suser.idm_tree_name,
    suser.is_curated,
    suser.job_code,
    suser.job_title,
    suser.last_name,
    suser.location,
    suser.merged,
    suser.middle_name,
    suser.office_phone,
    suser.other_phone,
    suser.preferred_locale,
    suser.provisioning_id,
    suser.secondary_supervisor_id,
    suser.supervisor_attr_key,
    suser.supervisor_id,
    suser.supervisor_unique_id,
    suser.unique_user_id,
    suser.user_id,
    suser.business_unit,
    suser.collected_entity,
    suser.effective_entity,
    suser.identity_source,
    suser.sversion,
    suser.workforce_id,
    suser_email.email,
    suser.risk
  FROM suser
    LEFT JOIN (SELECT
        suser.id suser,
        suser.display_name,
        string_agg(suser_email.email, ' | ' order by suser_email.email) As email
      FROM suser
        JOIN suser_email ON suser.id = suser_email.suser AND suser.effective = '1' AND suser.deleted = '0'   
      GROUP BY suser.id, suser.display_name) suser_email ON suser.id = suser_email.suser
  WHERE suser.deleted = '0' AND suser.effective = '1';

CREATE VIEW adhoc_sgroup_suser_v AS
  SELECT 
    sgroup_user.suser suser_id,
    sgroup.name group_name,
    sgroup.short_description,
    sgroup.id
  FROM sgroup_user
    LEFT JOIN sgroup ON sgroup_user.sgroup = sgroup.id
  WHERE sgroup.deleted = false AND sgroup.effective = true;

ALTER TABLE adhoc_sgroup_suser_v
  OWNER TO igops;
GRANT ALL ON TABLE adhoc_sgroup_suser_v TO igops;
GRANT SELECT ON TABLE adhoc_sgroup_suser_v TO ig_report_role;

ALTER TABLE suser_v
  OWNER TO igops;
GRANT ALL ON TABLE suser_v TO igops;
GRANT SELECT ON TABLE suser_v TO ig_report_role;

ALTER TABLE access_review_details_v
  OWNER TO igops;
GRANT ALL ON TABLE access_review_details_v TO igops;
GRANT SELECT ON TABLE access_review_details_v TO ig_report_role;

ALTER TABLE shared_accounts_v
  OWNER TO igops;
GRANT ALL ON TABLE shared_accounts_v TO igops;
GRANT SELECT ON TABLE shared_accounts_v TO ig_report_role;

ALTER TABLE review_item_status_v
  OWNER TO igops;
GRANT ALL ON TABLE review_item_status_v TO igops;
GRANT SELECT ON TABLE review_item_status_v TO ig_report_role;

ALTER TABLE adhoc_apps_spermissions_v
  OWNER TO igops;
GRANT ALL ON TABLE adhoc_apps_spermissions_v TO igops;
GRANT SELECT ON TABLE adhoc_apps_spermissions_v TO ig_report_role;

ALTER TABLE suser_spermission_cert_risk_v
  OWNER TO igops;
GRANT ALL ON TABLE suser_spermission_cert_risk_v TO igops;
GRANT SELECT ON TABLE suser_spermission_cert_risk_v TO ig_report_role;

ALTER TABLE adhoc_suser_v
  OWNER TO igops;
GRANT ALL ON TABLE adhoc_suser_v TO igops;
GRANT SELECT ON TABLE adhoc_suser_v TO ig_report_role;
